import { EntityInventoryComponent, ItemStack, MolangVariableMap, Player, system, world } from "@minecraft/server";
import { ModalFormData } from "@minecraft/server-ui";
let adminSettings = {
    activated: false,
    length: 2,
    font: 0,
    healColor: {
        red: 0,
        green: 1,
        blue: 0
    },
    damageColor: {
        red: 1,
        green: 0,
        blue: 0
    }
};
const loadedSettings = world.getDynamicProperty("adminSettings");
if (loadedSettings)
    adminSettings = JSON.parse(loadedSettings);
const defaultSettings = {
    activated: true,
    length: 2,
    font: 0,
    healColor: {
        red: 0,
        green: 1,
        blue: 0
    },
    damageColor: {
        red: 1,
        green: 0,
        blue: 0
    }
};
const Fonts = [
    "Default",
    "Mojangles"
];
const playerSettings = {};
world.getAllPlayers().forEach((player) => {
    let settings = defaultSettings;
    const loadedSettings = getData(player);
    if (loadedSettings)
        settings = loadedSettings;
    playerSettings[player.id] = settings;
});
function getData(player) {
    const data = player.getDynamicProperty("settings");
    return data ? JSON.parse(data) : defaultSettings;
}
function setData(data, player) {
    player.setDynamicProperty("settings", JSON.stringify(data));
}
system.afterEvents.scriptEventReceive.subscribe(async (data) => {
    if (data.id != "damagenumbers:admin_settings")
        return;
    if (!(data.sourceEntity instanceof Player))
        return;
    const player = data.sourceEntity;
    const settings = adminSettings;
    const ui = new ModalFormData();
    ui.title(`Global Damage Numbers Settings`);
    ui.toggle("Activated", settings.activated);
    ui.slider("Number Lifetime", 1, 10, 1, settings.length);
    ui.dropdown("Font", Fonts, settings.font ? settings.font : 0);
    ui.slider("§aHeal §rColor §cR", 0, 10, 1, settings.healColor?.red !== undefined ? settings.healColor.red * 10 : defaultSettings.healColor.red);
    ui.slider("§aHeal §rColor §aG", 0, 10, 1, settings.healColor?.green !== undefined ? settings.healColor.green * 10 : defaultSettings.healColor.green);
    ui.slider("§aHeal §rColor §9B", 0, 10, 1, settings.healColor?.blue !== undefined ? settings.healColor.blue * 10 : defaultSettings.healColor.blue);
    ui.slider("§cDamage §rColor §cR", 0, 10, 1, settings.damageColor?.red !== undefined ? settings.damageColor.red * 10 : defaultSettings.damageColor.red);
    ui.slider("§cDamage §rColor §aG", 0, 10, 1, settings.damageColor?.green !== undefined ? settings.damageColor.green * 10 : defaultSettings.damageColor.green);
    ui.slider("§cDamage §rColor §9B", 0, 10, 1, settings.damageColor?.blue !== undefined ? settings.damageColor.blue * 10 : defaultSettings.damageColor.blue);
    const response = await ui.show(player);
    if (response.canceled)
        return;
    if (!response.formValues)
        return;
    const newData = {
        activated: response.formValues[0],
        length: response.formValues[1],
        font: response.formValues[2],
        healColor: {
            red: response.formValues[3] / 10,
            green: response.formValues[4] / 10,
            blue: response.formValues[5] / 10
        },
        damageColor: {
            red: response.formValues[6] / 10,
            green: response.formValues[7] / 10,
            blue: response.formValues[8] / 10
        },
    };
    adminSettings = newData;
    world.setDynamicProperty("adminSettings", JSON.stringify(newData));
});
world.afterEvents.entityHealthChanged.subscribe((evdata) => {
    const { entity, newValue, oldValue } = evdata;
    const damage = Math.floor(newValue - oldValue);
    if (!entity || !entity.isValid())
        return;
    const loc = entity.location;
    loc.y += 0.5;
    const players = entity.dimension.getEntities({ type: "player", location: loc, maxDistance: 64 });
    for (const player of players) {
        let data = playerSettings[player.id];
        if (adminSettings?.activated)
            data = adminSettings;
        if (!data?.activated)
            return;
        const viewDir = player.getViewDirection();
        loc.x += -viewDir.x;
        loc.z += -viewDir.z;
        const rot = player.getRotation();
        const molang = new MolangVariableMap();
        let absDamage = Math.abs(damage);
        if (absDamage > 999999)
            absDamage = 999999;
        molang.setFloat("variable.length", data.length);
        molang.setFloat("variable.damage", damage);
        molang.setFloat("variable.roty", rot.y);
        molang.setFloat("variable.digits", `${absDamage}`.length);
        molang.setFloat("variable.floored", (absDamage % 10));
        molang.setFloat("variable.floored_tenths", (Math.floor(absDamage / 10) % 10));
        molang.setFloat("variable.floored_hundreths", (Math.floor(absDamage / 100) % 10));
        molang.setFloat("variable.floored_thousandths", (Math.floor(absDamage / 1000) % 10));
        molang.setFloat("variable.floored_ten_thousandths", (Math.floor(absDamage / 10000) % 10));
        molang.setFloat("variable.floored_hundred_thousandths", (Math.floor(absDamage / 100000) % 10));
        molang.setFloat("variable.font", data.font ? data.font : 0);
        molang.setColorRGB("variable.healcolor", data.healColor ? data.healColor : defaultSettings.healColor);
        molang.setColorRGB("variable.damagecolor", data.damageColor ? data.damageColor : defaultSettings.damageColor);
        try {
            player.spawnParticle("damagenumbers:damage_number", loc, molang);
        }
        catch { }
    }
});
world.afterEvents.playerJoin.subscribe((data) => {
    const player = world.getEntity(data.playerId);
    if (!player)
        return;
    let settings = defaultSettings;
    const loadedSettings = getData(player);
    if (loadedSettings)
        settings = loadedSettings;
    playerSettings[player.id] = settings;
    if (adminSettings.activated)
        return;
    if (player.getDynamicProperty("joined"))
        return;
    const inv = player.getComponent(EntityInventoryComponent.componentId)?.container;
    if (!inv)
        return;
    player.setDynamicProperty("joined", true);
    const item = new ItemStack("damagenumbers:settings", 1);
    if (inv.emptySlotsCount > 0) {
        inv.addItem(item);
    }
    else {
        const playerLoc = player.location;
        const itemEntity = player.dimension.spawnItem(item, { x: playerLoc.x, y: 100, z: playerLoc.z });
        itemEntity.teleport(playerLoc);
    }
});
world.afterEvents.itemUse.subscribe(async (data) => {
    if (data.itemStack.typeId != "damagenumbers:settings")
        return;
    const settings = getData(data.source);
    const ui = new ModalFormData();
    ui.title(`${data.source.name}${data.source.name.endsWith("s") ? "'" : "'s"} Damage Numbers Settings`);
    ui.toggle("Activated", settings.activated);
    ui.slider("Number Lifetime", 1, 10, 1, settings.length);
    ui.dropdown("Font", Fonts, settings.font ? settings.font : 0);
    ui.slider("§aHeal §rColor §cR", 0, 10, 1, settings.healColor?.red !== undefined ? settings.healColor.red * 10 : defaultSettings.healColor.red);
    ui.slider("§aHeal §rColor §aG", 0, 10, 1, settings.healColor?.green !== undefined ? settings.healColor.green * 10 : defaultSettings.healColor.green);
    ui.slider("§aHeal §rColor §9B", 0, 10, 1, settings.healColor?.blue !== undefined ? settings.healColor.blue * 10 : defaultSettings.healColor.blue);
    ui.slider("§cDamage §rColor §cR", 0, 10, 1, settings.damageColor?.red !== undefined ? settings.damageColor.red * 10 : defaultSettings.damageColor.red);
    ui.slider("§cDamage §rColor §aG", 0, 10, 1, settings.damageColor?.green !== undefined ? settings.damageColor.green * 10 : defaultSettings.damageColor.green);
    ui.slider("§cDamage §rColor §9B", 0, 10, 1, settings.damageColor?.blue !== undefined ? settings.damageColor.blue * 10 : defaultSettings.damageColor.blue);
    const response = await ui.show(data.source);
    if (response.canceled)
        return;
    if (!response.formValues)
        return;
    const newData = {
        activated: response.formValues[0],
        length: response.formValues[1],
        font: response.formValues[2],
        healColor: {
            red: response.formValues[3] / 10,
            green: response.formValues[4] / 10,
            blue: response.formValues[5] / 10
        },
        damageColor: {
            red: response.formValues[6] / 10,
            green: response.formValues[7] / 10,
            blue: response.formValues[8] / 10
        },
    };
    playerSettings[data.source.id] = newData;
    setData(newData, data.source);
});
